@@ -4,14 +4,14 @@ import pprint
 
 from dateutil.relativedelta import *
 import ee
+import openet.core.interpolate
+# TODO: import utils from openet.core
+# import openet.core.utils as utils
 
 from . import utils
 from .image import Image
 # Importing to get version number, is there a better way?
 import openet.ssebop
-import openet.core.interpolate
-# TODO: import utils from openet.core
-# import openet.core.utils as utils
 
 
 def lazy_property(fn):
@@ -44,6 +44,7 @@ class Collection():
             et_reference_band=None,
             et_reference_factor=None,
             et_reference_resample=None,
+            et_reference_date_type=None,
             filter_args=None,
             model_args=None,
             # model_args={'et_reference_source': 'IDAHO_EPSCOR/GRIDMET',
@@ -116,6 +117,7 @@ class Collection():
         self.et_reference_band = et_reference_band
         self.et_reference_factor = et_reference_factor
         self.et_reference_resample = et_reference_resample
+        self.et_reference_date_type = et_reference_date_type
 
         # Check reference ET parameters
         if et_reference_factor and not utils.is_number(et_reference_factor):
@@ -126,6 +128,10 @@ class Collection():
         if (et_reference_resample and
                 et_reference_resample.lower() not in et_reference_resample_methods):
             raise ValueError('unsupported et_reference_resample method')
+        et_reference_date_type_methods = ['doy', 'daily']
+        if (et_reference_date_type and
+                et_reference_date_type.lower() not in et_reference_date_type_methods):
+            raise ValueError('unsupported et_reference_date_type method')
 
         # Set/update the reference ET parameters in model_args if they were set in init()
         if self.et_reference_source:
@@ -136,11 +142,23 @@ class Collection():
             self.model_args['et_reference_factor'] = self.et_reference_factor
         if self.et_reference_resample:
             self.model_args['et_reference_resample'] = self.et_reference_resample
+        if self.et_reference_date_type:
+            self.model_args['et_reference_date_type'] = self.et_reference_date_type
+        # elif self.et_reference_date_type is None:
+        #     # Extra conditional needed since None is currently a valid date_type
+        #     # This should probably be changed so that "daily" is the default
+        #     self.model_args['et_reference_date_type'] = None
 
         # Model specific variables that can be interpolated to a daily timestep
         # CGM - Should this be specified in the interpolation method instead?
         self._interp_vars = ['ndvi', 'et_fraction']
 
+        self._landsat_c2_sr_collections = [
+            'LANDSAT/LC08/C02/T1_L2',
+            'LANDSAT/LE07/C02/T1_L2',
+            'LANDSAT/LT05/C02/T1_L2',
+            'LANDSAT/LT04/C02/T1_L2',
+        ]
         self._landsat_c1_sr_collections = [
             'LANDSAT/LC08/C01/T1_SR',
             'LANDSAT/LE07/C01/T1_SR',
@@ -162,7 +180,8 @@ class Collection():
 
         # Check that collection IDs are supported
         for coll_id in self.collections:
-            if (coll_id not in self._landsat_c1_sr_collections and
+            if (coll_id not in self._landsat_c2_sr_collections and
+                    coll_id not in self._landsat_c1_sr_collections and
                     coll_id not in self._landsat_c1_toa_collections):
                 raise ValueError('unsupported collection: {}'.format(coll_id))
 
@@ -251,14 +270,16 @@ class Collection():
         # Build the variable image collection
         variable_coll = ee.ImageCollection([])
         for coll_id in self.collections:
-            # DEADBEEF - Move to separate methods/functions for each type
-            if coll_id in self._landsat_c1_toa_collections:
+            # TODO: Move to separate methods/functions for each collection type
+            if coll_id in self._landsat_c2_sr_collections:
                 input_coll = ee.ImageCollection(coll_id)\
                     .filterDate(start_date, end_date)\
                     .filterBounds(self.geometry)\
-                    .filterMetadata('DATA_TYPE', 'equals', 'L1TP')\
                     .filterMetadata('CLOUD_COVER_LAND', 'less_than',
-                                    self.cloud_cover_max)
+                                    self.cloud_cover_max)\
+                    .filterMetadata('CLOUD_COVER_LAND', 'greater_than', -0.5)
+                # TODO: Check if PROCESSING_LEVEL needs to be filtered on
+                #     .filterMetadata('PROCESSING_LEVEL', 'equals', 'L2SP')
 
                 # TODO: Need to come up with a system for applying
                 #   generic filter arguments to the collections
@@ -271,13 +292,51 @@ class Collection():
                         if filter_type.lower() == 'equals':
                             input_coll = input_coll.filter(ee.Filter.equals(**f))
 
+                # TODO: Check if these bad images are in collection 2
                 # Time filters are to remove bad (L5) and pre-op (L8) images
                 if 'LT05' in coll_id:
                     input_coll = input_coll.filter(ee.Filter.lt(
                         'system:time_start', ee.Date('2011-12-31').millis()))
                 elif 'LC08' in coll_id:
                     input_coll = input_coll.filter(ee.Filter.gt(
-                        'system:time_start', ee.Date('2013-03-24').millis()))
+                        'system:time_start', ee.Date('2013-04-01').millis()))
+
+                def compute_lsr(image):
+                    model_obj = Image.from_landsat_c2_sr(
+                        sr_image=ee.Image(image), **self.model_args)
+                    return model_obj.calculate(variables)
+
+                variable_coll = variable_coll.merge(
+                    ee.ImageCollection(input_coll.map(compute_lsr)))
+
+            elif coll_id in self._landsat_c1_toa_collections:
+                input_coll = ee.ImageCollection(coll_id)\
+                    .filterDate(start_date, end_date)\
+                    .filterBounds(self.geometry)\
+                    .filterMetadata('DATA_TYPE', 'equals', 'L1TP')\
+                    .filterMetadata('CLOUD_COVER_LAND', 'less_than',
+                                    self.cloud_cover_max)\
+                    .filterMetadata('CLOUD_COVER_LAND', 'greater_than', -0.5)
+
+                # TODO: Need to come up with a system for applying
+                #   generic filter arguments to the collections
+                if coll_id in self.filter_args.keys():
+                    for f in copy.deepcopy(self.filter_args[coll_id]):
+                        try:
+                            filter_type = f.pop('type')
+                        except KeyError:
+                            continue
+                        if filter_type.lower() == 'equals':
+                            input_coll = input_coll.filter(ee.Filter.equals(**f))
+
+                # TODO: Check if these bad images are in collection 1 SR
+                # Time filters are to remove bad (L5) and pre-op (L8) images
+                if 'LT05' in coll_id:
+                    input_coll = input_coll.filter(ee.Filter.lt(
+                        'system:time_start', ee.Date('2011-12-31').millis()))
+                elif 'LC08' in coll_id:
+                    input_coll = input_coll.filter(ee.Filter.gt(
+                        'system:time_start', ee.Date('2013-04-01').millis()))
 
                 def compute_ltoa(image):
                     model_obj = Image.from_landsat_c1_toa(
@@ -292,7 +351,8 @@ class Collection():
                     .filterDate(start_date, end_date)\
                     .filterBounds(self.geometry)\
                     .filterMetadata('CLOUD_COVER_LAND', 'less_than',
-                                    self.cloud_cover_max)
+                                    self.cloud_cover_max)\
+                    .filterMetadata('CLOUD_COVER_LAND', 'greater_than', -0.5)
 
                 # TODO: Need to come up with a system for applying
                 #   generic filter arguments to the collections
@@ -311,7 +371,7 @@ class Collection():
                         'system:time_start', ee.Date('2011-12-31').millis()))
                 elif 'LC08' in coll_id:
                     input_coll = input_coll.filter(ee.Filter.gt(
-                        'system:time_start', ee.Date('2013-03-24').millis()))
+                        'system:time_start', ee.Date('2013-04-01').millis()))
 
                 def compute_lsr(image):
                     model_obj = Image.from_landsat_c1_sr(
@@ -454,6 +514,9 @@ class Collection():
         if ('et_reference_resample' in kwargs.keys() and \
                 kwargs['et_reference_resample'] is not None):
             self.model_args['et_reference_resample'] = kwargs['et_reference_resample']
+        if ('et_reference_date_type' in kwargs.keys() and \
+                kwargs['et_reference_date_type'] is not None):
+            self.model_args['et_reference_date_type'] = kwargs['et_reference_date_type']
 
         # Check that all reference ET parameters were set
         # print(self.model_args)
@@ -467,14 +530,35 @@ class Collection():
         if type(self.model_args['et_reference_source']) is str:
             # Assume a string source is an single image collection ID
             #   not an list of collection IDs or ee.ImageCollection
-            daily_et_ref_coll_id = self.model_args['et_reference_source']
-            daily_et_ref_coll = ee.ImageCollection(daily_et_ref_coll_id)\
-                .filterDate(start_date, end_date)\
-                .select([self.model_args['et_reference_band']], ['et_reference'])
+            if ('et_reference_date_type' not in self.model_args.keys() or
+                    self.model_args['et_reference_date_type'] is None or
+                    self.model_args['et_reference_date_type'].lower() == 'daily'):
+                daily_et_ref_coll = ee.ImageCollection(self.model_args['et_reference_source'])\
+                    .filterDate(start_date, end_date)\
+                    .select([self.model_args['et_reference_band']], ['et_reference'])
+            elif self.model_args['et_reference_date_type'].lower() == 'doy':
+                # Assume the image collection is a climatology with a "DOY" property
+                def doy_image(input_img):
+                    """Return the doy-based reference et with daily time properties from GRIDMET"""
+                    image_date = ee.Algorithms.Date(input_img.get('system:time_start'))
+                    image_doy = ee.Number(image_date.getRelative('day', 'year')).add(1).int()
+                    doy_coll = ee.ImageCollection(self.model_args['et_reference_source'])\
+                        .filterMetadata('DOY', 'equals', image_doy)\
+                        .select([self.model_args['et_reference_band']], ['et_reference'])
+                    # CGM - Was there a reason to use rangeContains if limiting to one DOY?
+                    #     .filter(ee.Filter.rangeContains('DOY', image_doy, image_doy))\
+                    return ee.Image(doy_coll.first())\
+                        .set({'system:index': input_img.get('system:index'),
+                              'system:time_start': input_img.get('system:time_start')})
+                # Note, the collection and band that are used are important as
+                #   long as they are daily and available for the time period
+                daily_et_ref_coll = ee.ImageCollection('IDAHO_EPSCOR/GRIDMET')\
+                    .filterDate(start_date, end_date).select(['eto'])\
+                    .map(doy_image)
         # elif isinstance(self.model_args['et_reference_source'], computedobject.ComputedObject):
         #     # Interpret computed objects as image collections
-        #     daily_et_ref_coll = self.model_args['et_reference_source'] \
-        #         .filterDate(self.start_date, self.end_date) \
+        #     daily_et_ref_coll = self.model_args['et_reference_source']\
+        #         .filterDate(self.start_date, self.end_date)\
         #         .select([self.model_args['et_reference_band']])
         else:
             raise ValueError('unsupported et_reference_source: {}'.format(
